# required packages TreeSim (simulating trees), FossilSim (simulating taxa) and fbdR (estimating speciation and extinction rates)

library(ggplot2)
library(gridExtra)

start.time = Sys.time()

source("../simulation_tests.R")

cat(NULL, file="log.txt", append = F)

set.seed(1234)

if(print.data)
  system("mkdir data")

# simulate trees 1
lambda.branching = 0.2 # branching rate
mu.branching = 0.02 # death rate

trees1<-TreeSim::sim.rateshift.taxa(tips,replicates,
                                    lambda=c(lambda.branching,0.3,lambda.branching),
                                    mu=c(mu.branching,0,mu.branching),
                                    frac=c(1,1,1),
                                    times=c(0,10,20))

if(print.data)
  print.trees(trees1, 1)

f = 0 # probability of symmetric speciation, if f = 0 all speciation events will be asymmetric, if f = 1 all speciation events will be symmetric
lambda.a = 0 # rate of anagenic speciation

# figure labels
lb1 = "list(lambda==0.2,0.3,0.2, mu==0.02,0,0.02)"
lb2 = "list(beta==0, lambda[a]==0)"
lb3 = "list(lambda~\"*\"==0.2,0.3,0.2, mu~\"*\"==0.02,0,0.02)"

# simulate ranges
ranges = simulate.ranges(trees1)

if(print.data)
  print.ranges(ranges, 1)

# likelihood ratio tests
out = likelihood.ratio.tests(trees1, ranges, "out1.log", shifts = TRUE)

# figures + ouput
p1 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d1 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c1 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o1 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out1 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 1...\n", file="log.txt", append=T)

f = 0.5
lambda.a = 0.04

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.02,0,0.02)"
lb2 = "list(beta==0.5, lambda[a]==0.04)"
lb3 = "list(lambda~\"*\"==0.34,0.49,0.34, mu~\"*\"==0.16,0.19,0.16)"

ranges = simulate.ranges(trees1)

if(print.data)
  print.ranges(ranges, 2)

out = likelihood.ratio.tests(trees1, ranges, "out2.log", shifts = TRUE)

p2 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d2 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c2 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o2 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out2 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 2...\n", file="log.txt", append=T)

f = 0.7
lambda.a = 0.16

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.02,0,0.02)"
lb2 = "list(beta==0.7, lambda[a]==0.16)"
lb3 = "list(lambda~\"*\"==0.5,0.67,0.5, mu~\"*\"==0.32,0.37,0.32)"

ranges = simulate.ranges(trees1)

if(print.data)
  print.ranges(ranges, 3)

out = likelihood.ratio.tests(trees1, ranges, "out3.log", shifts = TRUE)

p3 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d3 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c3 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o3 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out3 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 3...\n", file="log.txt", append=T)

# simulate trees 2
lambda.branching = 0.2 # branching rate
mu.branching = 0.1 # death rate

trees2<-TreeSim::sim.rateshift.taxa(tips,replicates,
                                    lambda=c(lambda.branching,0.3,lambda.branching),
                                    mu=c(mu.branching,0,mu.branching),
                                    frac=c(1,1,1),
                                    times=c(0,10,20))
if(print.data)
  print.trees(trees2, 2)

f = 0
lambda.a = 0 # rate of anagenic speciation

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.1,0,0.1)"
lb2 = "list(beta==0, lambda[a]==0)"
lb3 = "list(lambda~\"*\"==0.2,0.3,0.2, mu~\"*\"==0.1,0,0.1)"

ranges = simulate.ranges(trees2)

if(print.data)
  print.ranges(ranges, 4)

out = likelihood.ratio.tests(trees2, ranges, "out4.log", shifts = TRUE)

p4 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d4 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c4 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o4 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out4 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 4...\n", file="log.txt", append=T)

f = 0.5
lambda.a = 0.04

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.1,0,0.1)"
lb2 = "list(beta==0.5, lambda[a]==0.04)"
lb3 = "list(lambda~\"*\"==0.34,0.49,0.34, mu~\"*\"==0.24,0.19,0.24)"

ranges = simulate.ranges(trees2)

if(print.data)
  print.ranges(ranges, 5)

out = likelihood.ratio.tests(trees2, ranges, "out5.log", shifts = TRUE)

p5 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d5 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c5 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o5 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out5 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 5...\n", file="log.txt", append=T)

f = 0.7
lambda.a = 0.16

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.1,0,0.1)"
lb2 = "list(beta==0.7, lambda[a]==0.16)"
lb3 = "list(lambda~\"*\"==0.5,0.67,0.5, mu~\"*\"==0.4,0.37,0.4)"

ranges = simulate.ranges(trees2)

if(print.data)
  print.ranges(ranges, 6)

out = likelihood.ratio.tests(trees2, ranges, "out6.log", shifts = TRUE)

p6 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d6 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c6 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o6 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out6 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 6...\n", file="log.txt", append=T)

# simulate trees 3
lambda.branching = 0.2 # branching rate
mu.branching = 0.16 # death rate

trees3<-TreeSim::sim.rateshift.taxa(tips,replicates,
                                    lambda=c(lambda.branching,0.3,lambda.branching),
                                    mu=c(mu.branching,0,mu.branching),
                                    frac=c(1,1,1),
                                    times=c(0,10,20))
if(print.data)
  print.trees(trees3, 3)

f = 0
lambda.a = 0 # rate of anagenic speciation

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.16,0,0.16)"
lb2 = "list(beta==0, lambda[a]==0)"
lb3 = "list(lambda~\"*\"==0.2,0.3,0.2, mu~\"*\"==0.16,0,0.16)"

ranges = simulate.ranges(trees3)

if(print.data)
  print.ranges(ranges, 7)

out = likelihood.ratio.tests(trees3, ranges, "out7.log", shifts = TRUE)

p7 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d7 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c7 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o7 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out7 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 7...\n", file="log.txt", append=T)

f = 0.5
lambda.a = 0.04

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.16,0,0.16)"
lb2 = "list(beta==0.5, lambda[a]==0.04)"
lb3 = "list(lambda~\"*\"==0.34,0.49,0.34, mu~\"*\"==0.3,0.19,0.3)"

ranges = simulate.ranges(trees3)

if(print.data)
  print.ranges(ranges, 8)

out = likelihood.ratio.tests(trees3, ranges, "out8.log", shifts = TRUE)

p8 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d8 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c8 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o8 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out8 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 8...\n", file="log.txt", append=T)

f = 0.7
lambda.a = 0.16

lb1 = "list(lambda==0.2,0.3,0.2, mu==0.16,0,0.16)"
lb2 = "list(beta==0.7, lambda[a]==0.16)"
lb3 = "list(lambda~\"*\"==0.5,0.67,0.5, mu~\"*\"==0.46,0.37,0.46)"

ranges = simulate.ranges(trees3)

if(print.data)
  print.ranges(ranges, 9)

out = likelihood.ratio.tests(trees3, ranges, "out9.log", shifts = TRUE)

p9 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d9 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c9 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o9 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out9 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 9...\n", file="log.txt", append=T)


##### output

# fig 1

pdf(file="simulations-rate-shift-estimates.pdf",width=12,height=12)
grid.arrange(p1,p4,p7,p2,p5,p8,p3,p6,p9,ncol=3)
dev.off()

# fig 2

pdf(file="simulations-rate-shift-correlations3.pdf",width=12,height=12)
grid.arrange(d1,d4,d7,d2,d5,d8,d3,d6,d9,ncol=3)
dev.off()

# fig 3

pdf(file="simulations-rate-shift-correlations1.pdf",width=12,height=12)
grid.arrange(c1,c4,c7,c2,c5,c8,c3,c6,c9,ncol=3)
dev.off()

# fig 4

pdf(file="simulations-rate-shift-correlations2.pdf",width=12,height=12)
grid.arrange(o1,o4,o7,o2,o5,o8,o3,o6,o9,ncol=3)
dev.off()

# summary file

print.headers()
print.results(out1)
print.results(out4)
print.results(out7)
print.results(out2)
print.results(out5)
print.results(out8)
print.results(out3)
print.results(out6)
print.results(out9)

### 

end.time = Sys.time() - start.time

sink("log.txt", append = T)
cat(end.time, "\n", sep = "", file="log.txt", append=T)
sink()

