# required packages TreeSim (simulating trees), FossilSim (simulating taxa) and fbdR (estimating speciation and extinction rates)

library(ggplot2)
library(gridExtra)

start.time = Sys.time()

source("../simulation_tests.R")

cat(NULL, file="log4.txt", append = F)

set.seed(1234)

if(print.data)
  system("mkdir data")

remove = 1.0
lb3 = "list(x==1.0)"

# simulate trees 1
lambda.branching = 0.2 # branching rate
mu.branching = 0.02 # death rate
trees1<-TreeSim::sim.bd.taxa(tips, replicates, lambda.branching, mu.branching)

if(print.data)
  print.trees(trees1, 1)

f = 0 # probability of symmetric speciation
lambda.a = 0 # rate of anagenic speciation

# figure labels
lb1 = "list(lambda==0.2, mu==0.02, beta==0, lambda[a]==0)" 
lb2 = "list(lambda~\"*\"==0.2,mu~\"*\"==0.02)"

# simulate ranges
ranges = simulate.ranges(trees1)

# remove ranges
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_1")

# likelihood ratio tests
out = likelihood.ratio.tests(trees1, ranges, "out1_4.log", file2 = "log4.txt")

# figures + ouput
p1 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d1 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c1 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o1 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out1 = out

# check for convergence issues
convergence.issues(out, file="log4.txt")

cat("finished 1...\n", file="log4.txt", append=T)

f = 0.5
lambda.a = 0.04

lb1 = "list(lambda==0.2, mu==0.02, beta==0.5, lambda[a]==0.04)"
lb2 = "list(lambda~\"*\"==0.34,mu~\"*\"==0.16)"

ranges = simulate.ranges(trees1)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_2")

out = likelihood.ratio.tests(trees1, ranges, "out2_4.log", file2 = "log4.txt")

p2 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d2 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c2 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o2 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out2 = out

convergence.issues(out, file="log4.txt")

cat("finished 2...\n",file="log4.txt", append=T)

f = 0.7
lambda.a = 0.16

lb1 = "list(lambda==0.2, mu==0.02, beta==0.7, lambda[a]==0.16)"
lb2 = "list(lambda~\"*\"==0.5,mu~\"*\"==0.32)"

ranges = simulate.ranges(trees1)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_3")

out = likelihood.ratio.tests(trees1, ranges, "out3_4.log", file2 = "log4.txt")

p3 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d3 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c3 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o3 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out3 = out

convergence.issues(out, file="log4.txt")

cat("finished 3...\n",file="log4.txt", append=T)

# simulate trees 2
lambda.branching = 0.2 # branching rate
mu.branching =0.1 # death rate
trees2<-TreeSim::sim.bd.taxa(tips, replicates, lambda.branching, mu.branching)

if(print.data)
  print.trees(trees2, 2)

f = 0
lambda.a = 0 # rate of anagenic speciation

lb1 = "list(lambda==0.2, mu==0.1, beta==0, lambda[a]==0)"
lb2 = "list(lambda~\"*\"==0.2,mu~\"*\"==0.1)"

ranges = simulate.ranges(trees2)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_4")

out = likelihood.ratio.tests(trees2, ranges, "out4_4.log", file2 = "log4.txt")

p4 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d4 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c4 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o4 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out4 = out

convergence.issues(out, file="log4.txt")

cat("finished 4...\n", file="log4.txt", append=T)

f = 0.5
lambda.a = 0.04

lb1 = "list(lambda==0.2, mu==0.1, beta==0.5, lambda[a]==0.04)"
lb2 = "list(lambda~\"*\"==0.34,mu~\"*\"==0.24)"

ranges = simulate.ranges(trees2)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_5")

out = likelihood.ratio.tests(trees2, ranges, "out5_4.log", file2 = "log4.txt")

p5 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d5 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c5 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o5 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out5 = out

convergence.issues(out5, file="log4.txt")

cat("finished 5...\n", file="log4.txt", append=T)

f = 0.7
lambda.a = 0.16

lb1 = "list(lambda==0.2, mu==0.1, beta==0.7, lambda[a]==0.16)"
lb2 = "list(lambda~\"*\"==0.5,mu~\"*\"==0.4)"

ranges = simulate.ranges(trees2)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_6")

out = likelihood.ratio.tests(trees2, ranges, "out6_4.log", file2 = "log4.txt")

p6 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d6 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c6 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o6 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out6 = out

convergence.issues(out, file="log4.txt")

cat("finished 6...\n", file="log4.txt", append=T)

# simulate trees 3
lambda.branching = 0.2 # branching rate
mu.branching = 0.16 # death rate
trees3<-TreeSim::sim.bd.taxa(tips, replicates, lambda.branching, mu.branching)

if(print.data)
  print.trees(trees3, 3)

f = 0
lambda.a = 0 # rate of anagenic speciation

lb1 = "list(lambda==0.2, mu==0.16, beta==0, lambda[a]==0)"
lb2 = "list(lambda~\"*\"==0.2,mu~\"*\"==0.16)"

ranges = simulate.ranges(trees3)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_7")

out = likelihood.ratio.tests(trees3, ranges, "out7_4.log", file2 = "log4.txt")

p7 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d7 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c7 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o7 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out7 = out

convergence.issues(out, file="log4.txt")

cat("finished 7...\n", file="log4.txt", append=T)

f = 0.5
lambda.a = 0.04

lb1 = "list(lambda==0.2, mu==0.16, beta==0.5, lambda[a]==0.04)"
lb2 = "list(lambda~\"*\"==0.34,mu~\"*\"==0.3)"

ranges = simulate.ranges(trees3)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_8")

out = likelihood.ratio.tests(trees3, ranges, "out8_4.log", file2 = "log4.txt")

p8 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d8 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c8 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o8 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out8 = out

convergence.issues(out, file="log4.txt")

cat("finished 8...\n", file="log4.txt", append=T)

f = 0.7
lambda.a = 0.16

lb1 = "list(lambda==0.2, mu==0.16, beta==0.7, lambda[a]==0.16)"
lb2 = "list(lambda~\"*\"==0.5,mu~\"*\"==0.46)"

ranges = simulate.ranges(trees3)
ranges = non.randomly.remove(ranges, remove)

if(print.data)
  print.ranges(ranges, "4_9")

out = likelihood.ratio.tests(trees3, ranges, "out9_4.log", file2 = "log4.txt")

p9 = estimates.figure(out, lb1, lb2, lb3 = lb3)
d9 = correlations.figure(out, lb1, lb2, lb3 = lb3)
c9 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 2)
o9 = correlations.figure(out, lb1, lb2, lb3 = lb3, equation = 3)
out9 = out

convergence.issues(out, file="log4.txt")

cat("finished 9...\n", file="log4.txt", append=T)

##### output

# fig 1

pdf(file="simulations-estimates-non-random-missing4.pdf",width=12,height=12)
grid.arrange(p1,p4,p7,p2,p5,p8,p3,p6,p9,ncol=3)
dev.off()

# fig 2

pdf(file="simulations-correlations3-non-random-missing4.pdf",width=12,height=12)
grid.arrange(d1,d4,d7,d2,d5,d8,d3,d6,d9,ncol=3)
dev.off()

# fig 3

pdf(file="simulations-correlations1-non-random-missing4.pdf",width=12,height=12)
grid.arrange(c1,c4,c7,c2,c5,c8,c3,c6,c9,ncol=3)
dev.off()

# fig 4

pdf(file="simulations-correlations2-non-random-missing4.pdf",width=12,height=12)
grid.arrange(o1,o4,o7,o2,o5,o8,o3,o6,o9,ncol=3)
dev.off()

# summary file

file1 = "lk_ratio_test_results_simulations4.txt"
#file2 = "lk_ratio_test_results_simulations4.tex"

print.headers(file = file1)
print.results(out1, file = file1)
print.results(out4, file = file1)
print.results(out7, file = file1)
print.results(out2, file = file1)
print.results(out5, file = file1)
print.results(out8, file = file1)
print.results(out3, file = file1)
print.results(out6, file = file1)
print.results(out9, file = file1)


end.time = Sys.time() - start.time

sink("log4.txt", append = T)
cat(end.time, "\n", sep = "", file="log4.txt", append=T)
sink()




