# required packages TreeSim (simulating trees), FossilSim (simulating taxa) and fbdR (estimating speciation and extinction rates)

start.time = Sys.time()

source("../simulation_tests.R")

cat(NULL, file="log2.txt", append = F)

set.seed(1234)

psi = 0.5

# simulate trees 1
lambda.branching = 1/5 # branching rate
mu.branching = 0.1/5 # death rate
trees1<-TreeSim::sim.bd.taxa(tips, replicates, lambda.branching, mu.branching)

system("mkdir trees1_extant")
parse.trees(trees1, prefix = "trees1_extant/tree1", extant.only = TRUE)

# simulate trees 2
lambda.branching = 1/5 # branching rate
mu.branching = 0.5/5 # death rate
trees2<-TreeSim::sim.bd.taxa(tips, replicates, lambda.branching, mu.branching)

system("mkdir trees2_extant")
parse.trees(trees2, prefix = "trees2_extant/tree2", extant.only = TRUE)

# simulate trees 3
lambda.branching = 1/5 # branching rate
mu.branching = 0.8/5 # death rate
trees3<-TreeSim::sim.bd.taxa(tips, replicates, lambda.branching, mu.branching)

system("mkdir trees3_extant")
parse.trees(trees3, prefix = "trees3_extant/tree3", extant.only = TRUE)

f = 0 # probability of symmetric speciation
lambda.a = 0 # rate of anagenic speciation

# simulate ranges
ranges = simulate.ranges(trees1)

# generate occurrences
occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate1")
parse.pyrate(occurrences, "pyrate1/pyrate1")

cat("finished 1...\n", file="log2.txt", append=T)

f = 0.5
lambda.a = 0.2/5

ranges = simulate.ranges(trees2)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate5")
parse.pyrate(occurrences, "pyrate5/pyrate5")

cat("finished 5...\n", file="log2.txt", append=T)

f = 0.7
lambda.a = 0.8/5

ranges = simulate.ranges(trees3)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate9")
parse.pyrate(occurrences, "pyrate9/pyrate9")

cat("finished 9...\n", file="log2.txt", append=T)

f = 0.5
lambda.a = 0.2/5

ranges = simulate.ranges(trees1)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate2")
parse.pyrate(occurrences, "pyrate2/pyrate2")

cat("finished 2...\n",file="log2.txt", append=T)

f = 0.7
lambda.a = 0.8/5

ranges = simulate.ranges(trees1)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate3")
parse.pyrate(occurrences, "pyrate3/pyrate3")

cat("finished 3...\n",file="log2.txt", append=T)

f = 0
lambda.a = 0 # rate of anagenic speciation

ranges = simulate.ranges(trees2)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate4")
parse.pyrate(occurrences, "pyrate4/pyrate4")

f = 0.7
lambda.a = 0.8/5

ranges = simulate.ranges(trees2)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate6")
parse.pyrate(occurrences, "pyrate6/pyrate6")

f = 0
lambda.a = 0 # rate of anagenic speciation

ranges = simulate.ranges(trees3)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate7")
parse.pyrate(occurrences, "pyrate7/pyrate7")

cat("finished 7...\n", file="log2.txt", append=T)

f = 0.5
lambda.a = 0.2/5

ranges = simulate.ranges(trees3)

occurrences = poisson.occurrences(ranges, psi)
system("mkdir pyrate8")
parse.pyrate(occurrences, "pyrate8/pyrate8")

cat("finished 8...\n", file="log2.txt", append=T)

