# required packages TreeSim (simulating trees), FossilSim (simulating taxa) and fbdR (estimating speciation and extinction rates)

library(ggplot2)
library(gridExtra)

start.time = Sys.time()

source("/Users/danielesilvestro/Software/speciesconcepts/simulations/version2/simulation_tests_opt.R")

setwd("/Users/danielesilvestro/Desktop/try/rate_var_incompatible")

cat(NULL, file="log.txt", append = F)

set.seed(1234)

if(print.data)
  system("mkdir data")

# simulate trees 1
lambda.branching = 1 # branching rate
mu.branching = 0.1 # death rate
replicates =100

print("simulating trees")
trees1<-TreeSim::sim.rateshift.taxa(tips,replicates,
                                    lambda=c(lambda.branching,lambda.branching,1.5),
                                    mu=c(1.5,mu.branching,mu.branching),
                                    frac=c(1,1,1),
                                    times=c(0,3,5))

trees2<-TreeSim::sim.bd.taxa(tips,replicates,
                             lambda=c(lambda.branching),
                             mu=c(mu.branching))


print("done")
#if(print.data)
#  print.trees(trees1, 1)

f = 0 # probability of symmetric speciation, if f = 0 all speciation events will be asymmetric, if f = 1 all speciation events will be symmetric
lambda.a = 0 # rate of anagenic speciation

# figure labels
lb1 = "list(lambda==1,1.5,1, mu==1,0,0.1)"
lb2 = "list(beta==0, lambda[a]==0)"
lb3 = "list(lambda~\"*\"==1,1.5,1, mu~\"*\"==1,0,0.1)"

ape::ltt.plot(trees2[[1]])
# simulate ranges
print("simulating ranges")
ranges = simulate.ranges(trees2)
print("done")

#if(print.data)
#  print.ranges(ranges, 1)

# likelihood ratio tests
out = likelihood.ratio.tests(trees1, ranges, "out1.log", shifts = TRUE)

# figures + ouput
p1 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d1 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c1 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o1 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out1 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 1...\n", file="log.txt", append=T)

f = 0.5
lambda.a = 0.2

lb1 = "list(lambda==1,1.5,1, mu==1,0,0.1)"
lb2 = "list(beta==0.5, lambda[a]==0.2)"
lb3 = "list(lambda~\"*\"==1.7,2.45,1.7, mu~\"*\"==0.8,0.95,0.8)"

print("simulating ranges")
ranges = simulate.ranges(trees2)
print("done")

if(print.data)
  print.ranges(ranges, 2)

out = likelihood.ratio.tests(trees1, ranges, "out2.log", shifts = TRUE)

p2 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d2 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c2 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o2 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out2 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 2...\n", file="log.txt", append=T)

f = 0.7
lambda.a = 0.8

lb1 = "list(lambda==1,1.5,1, mu==1,0,0.1)"
lb2 = "list(beta==0.7, lambda[a]==0.8)"
lb3 = "list(lambda~\"*\"==2.5,3.35,2.5, mu~\"*\"==1.6,1.85,1.6)"

print("simulating ranges")
ranges = simulate.ranges(trees2)
print("done")

if(print.data)
  print.ranges(ranges, 3)

out = likelihood.ratio.tests(trees1, ranges, "out3.log", shifts = TRUE)

p3 = estimates.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
d3 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE)
c3 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 2)
o3 = correlations.figure(out, lb1, lb2, lb3 = lb3, shifts = TRUE, equation = 3)
out3 = out

# check for convergence issues
convergence.issues(out, file="log.txt")

cat("finished 3...\n", file="log.txt", append=T)







##### output

# fig 1

pdf(file="simulations-rate-shift-estimates.pdf",width=12,height=4)
grid.arrange(p1,p2,p3,ncol=3)
dev.off()

# fig 2

pdf(file="simulations-rate-shift-correlations3.pdf",width=12,height=4)
grid.arrange(d1,d2,d3,ncol=3)
dev.off()

# fig 3

pdf(file="simulations-rate-shift-correlations1.pdf",width=12,height=4)
grid.arrange(c1,c2,c3,ncol=3)
dev.off()

# fig 4

pdf(file="simulations-rate-shift-correlations2.pdf",width=12,height=4)
grid.arrange(o1,o2,o3,ncol=3)
dev.off()

# summary file

print.headers()
print.results(out1)
print.results(out2)
print.results(out3)

### 

end.time = Sys.time() - start.time

sink("log.txt", append = T)
cat(end.time, "\n", sep = "", file="log.txt", append=T)
sink()

